local M = {}

local function init() 
	electrics.values["flaps"] = 0
	electrics.values["elevator"] = 0
	electrics.values["alt100"] = 0
	electrics.values["alt1000"] = 0
	electrics.values["thrust"] = 0
	electrics.values["yaw"] = 0
	electrics.values["roll"] = 0
	electrics.values["thrustrot"] = 0
	electrics.values['attitude'] = 0
	electrics.values['clocks'] = 0
	electrics.values['clockm'] = 0
	electrics.values['clockh'] = 0 
	electrics.values['kphn'] = 0 
	electrics.values['gear'] = 0 
	electrics.values['gear_door'] = 0
end

local function reset()
	init()
end


local function updateGFX(dt)
	--gear
	local gr = electrics.values["lowhighbeam"]
	local grl = gr
	local grr = gr
	if gr < 0 then gr = 0 end
	--hacky gear steering
	if gr < 1 then
		local grs = electrics.values["steering_input"]
		grl = grl - grs / 35
		grr = grr + grs / 35
	end
	
	electrics.values['gearL'] = grr
	electrics.values['gearR'] = grl
	electrics.values['gear'] = gr
	--kph
	local airspeed = electrics.values['airspeed']
	local kph = airspeed * 3.6
	kph = kph - 100
	if kph < 0 then kph = 0 end
	kph = kph / 1000
	electrics.values['kphn'] = kph
	--roll
	local dirVector = obj:getDirectionVector()
 	local dirVectorUp = obj:getDirectionVectorUp()
   	local roll = dirVectorUp.x * -dirVector.y + dirVectorUp.y * dirVector.x
	local rollProcessed = math.deg(roll) 
	electrics.values['attitude'] = rollProcessed * -1
	--yaw
	electrics.values["yaw"] = math.deg(obj:getDirection()) % 360 / 360
	--print(electrics.values["yaw"])
	--thrust
	electrics.values["thrust"] = thrusters.thrust()
	
	local tr = electrics.values["thrustrot"]
	tr = tr + (thrusters.thrust() * dt)
	tr = tr % 1
	electrics.values["thrustrot"] = tr
	--altitude
	local alt = obj:getPosition().z
	local a100 = (alt%100) / 100
	local a1000 = (alt%1000) / 1000
	electrics.values["alt1000"] = a1000
	electrics.values["alt100"] = a100
	--flaps and elevator
	local f = electrics.values["flaps"]
	if input.keys["H"] then f = f + (1 * dt) end
	if input.keys["Y"] then f = f - (1 * dt) end
	if f < 0 then f = 0 end
	if f > 1 then f = 1 end
	electrics.values["flaps"] = f
	electrics.values["elevator"] = (electrics.values["throttle"] - electrics.values["brake"]) * -1
	--clock
	local dateTime = os.date("*t", os.time())
	
	local psec = dateTime.sec/ 60
	local pmin = (dateTime.min + psec) / 60
	local ph = ((dateTime.hour % 12) + (pmin)) / 12
	
	
	--prop functions : st10clocks st10clockm st10clockh
	--prop factor should be 360
	electrics.values['clocks'] = psec
	electrics.values['clockm'] = pmin 
	electrics.values['clockh'] =  ph 
	

end

M.updateGFX = updateGFX
M.init = init
M.reset = reset
return M